#include "CommandSet.h"

CommandSet::CommandSet(int init_unique_id, const QStringList & init_commands)
 : unique_id(init_unique_id),
   commands(init_commands),
   status(UK1104::UK1104_PENDING) {
   return;
}

CommandSet::~CommandSet() {
   return;
}

int
CommandSet::GetUniqueId() {
   return unique_id;
}

UK1104::UK1104Errors
CommandSet::GetStatus() {
   return status;
}

QString
CommandSet::GetResults() {
   return results;
}
